% blt2.m - bilinear transformation of analog second-order sections
%
% Usage: [B,A,Bhat,Ahat] = blt2(Ba,Aa,w0,p);
%
% Ba,Aa = (L+1)x3 matrices of analog numerator and denominator coefficients (L 2nd-order sections)
% w0    = center frequency in radians/sample 
% p     = +1 for LP/HP/BP, p=-1 for BS
%
% B,A = (L+1)x5 matrices of numerator and denominator coefficients (4th-order sections in z)
% Bhat,Ahat = (L+1)x3 matrices of 2nd-order sections in the variable zhat
%
% notes: It implements the two-stage bilinear transformation: 
%
%                       s    -->     zhat      -->     z
%                  LP_analog --> LP/HP_digital --> BP/BS_digital
%
%        c0 = cos(w0), where w0 = 2*pi*f0/fs = center frequency in radians/sample
%
%        zhat = z*(c0-z)/(1-c0*z)
%
%        s = (zhat-1)/(zhat+1) = (z^2 - 2*c0*z + 1)/(z^2 - 1)  =  BP case, has p=+1
%        s = (zhat+1)/(zhat-1) = (z^2 - 1) /(z^2 - 2*c0*z + 1) =  BS case, has p=-1
%
%        (B0 + B1*s + B2*s^2)/(A0 + A1*s + A2*s^2) = 
%        (b0h + b1h*zhat^-1 + b2h*zhat^-2)/(1 + a1h*zhat^-1 + a2h*zhat^-2) =
%        (b0 + b1*z^-1 + b2*z^-2 + b3*z^-3 + b4*z^-4)/(1 + a1*z^-1 + a2*z^-2 + a3*z^-3 + a4*z^-4)
%
%        column-wise, the input and output matrices have the forms:
%        Ba = [B0,B1,B2], Bhat = [b0h, b1h, b2h], B = [b0,b1,b2,b3,b4]
%        Aa = [A0,A1,A2], Ahat = [1,   a1h, a2h], A = [1, a1,a2,a3,a4]
%
%        B,A are Kx3 matrices in the LP or HP cases
%
%        BLT2 is a modified version of BLT of the HPEQ toolbox that includes bandstop designs

% -------------------------------------------------------------------------
% Sophocles J. Orfanidis - September 2006
% Email:   orfanidi@ece.rutgers.edu
% 
% based on "Lecture Notes on Elliptic Filter Design" by Sophocles J. Orfanidis
% available from www.ece.rutgers.edu/~orfanidi/ece521
% -------------------------------------------------------------------------
        
function [B,A,Bhat,Ahat] = blt3(Ba,Aa,w0,p)
    
if nargin==0, help blt2; return; end
if nargin==3, p=1; end

L = size(Aa,1) - 1;         % number of 2nd order sections

c0 = cos(w0);       

if w0==0,    c0=1;  end;                       % make sure special cases are computed exactly
if w0==pi;   c0=-1; end;
if w0==pi/2; c0=0;  end;

if Aa(1,2)==0,                                 % even N case
   Bhat(1,:) = Ba(1,:);
   Ahat(1,:) = Aa(1,:);
   B(1,:) = [Bhat(1,:), 0, 0];
   A(1,:) = [Ahat(1,:), 0, 0];
else                                           % odd N case
   Bhat(1,:) = [1, p, 0]/(1 + Aa(1,2));
   Ahat(1,:) = [1, p*(1 - Aa(1,2))/(1 + Aa(1,2)), 0];
   B(1,:) = [Bhat(1,1), c0*(Bhat(1,2)-Bhat(1,1)), -Bhat(1,2), 0, 0];
   A(1,:) = [1, c0*(Ahat(1,2)-1), -Ahat(1,2), 0, 0];
end

i = (2:L+1)';                                           

D = 1 + Aa(i,2) + Aa(i,3);                              % second-order sections
Bhat(i,1) = (Ba(i,1) + Ba(i,2) + Ba(i,3))./D;
Bhat(i,2) = p * 2 * (Ba(i,1) - Ba(i,3))./D;             % change sign of z^-1 coeff for BS case
Bhat(i,3) = (Ba(i,1) - Ba(i,2) + Ba(i,3))./D;
Ahat(i,1) = 1;
Ahat(i,2) = p * 2 * (Aa(i,1) - Aa(i,3))./D;             % change sign of z^-1 coeff for BS case
Ahat(i,3) = (Aa(i,1) - Aa(i,2) + Aa(i,3))./D;

B(i,1) = Bhat(i,1);                                     % fourth-order sections
B(i,2) = c0*(Bhat(i,2)-2*Bhat(i,1));
B(i,3) = (Bhat(i,1)-Bhat(i,2)+Bhat(i,3))*c0^2 - Bhat(i,2);
B(i,4) = c0*(Bhat(i,2)-2*Bhat(i,3));
B(i,5) = Bhat(i,3);

A(i,1) = 1;
A(i,2) = c0*(Ahat(i,2)-2);
A(i,3) = (1-Ahat(i,2)+Ahat(i,3))*c0^2 - Ahat(i,2);
A(i,4) = c0*(Ahat(i,2)-2*Ahat(i,3));
A(i,5) = Ahat(i,3);

if c0==1 | c0==-1 	        % LP or HP cases
   B = Bhat;                    % B,A are (L+1)x3
   A = Ahat;
   B(:,2) = c0*B(:,2);	        % change sign if w0=pi
   A(:,2) = c0*A(:,2);
end    







    
    
        











