% dfdes.m  -  digital filter design with the bilinear transformation
% 
% Usage: [B,A,w0] = dfdes(N, Ad, wd, type, shape, coeffs)
%
%        [B,A,w0] = dfdes(N, Ad, wd, type, shape)           % equivalent to coeffs='4os'
%        [B,A,w0] = dfdes(N, Ad, wd, type, shape, '4os')    % fourth-order sections
%        [B,A,w0] = dfdes(N, Ad, wd, type, shape, 'hsos')   % hat-second-order sections
%        [B,A,w0] = dfdes(N, Ad, wd, type, shape, 'sos')    % second-order sections
%        [B,A,w0] = dfdes(N, Ad, wd, type, shape, 'dir')    % direct-form coefficients
%   
% N      = filter order of LP analog prototype, digital filter order is 2*N,  N=2L+r, r=0,1 
% Ad     = attenuation in dB at the design frequency wd, (Ad is 1d, 2d, or 3d row vector)
% wd     = design frequency in radians/sample, (wd is 1d for LP/HP, or, 2d for BP/BS)
% type   = 0,1,2,3 for Butterworth, Chebyshev-1, Chebyshev-2, Elliptic
% shape  = 'LP', 'HP', 'BP', 'BS', for lowpass, highpass, bandpass, bandstop designs
% coeffs = '4os', 'sos', 'hsos', 'dir', 4th-order sections, hat-2nd-order, 2nd-order, direct-form 
%
% B,A = (L+1)x5 for '4os', (L+1)x3 for 'hsos', (2L+1)x3 for 'sos', (2N+1)x1 for 'dir'
% w0  = center freq (in rads/sample) of the band wd for BP/BS, w0=0 for LP, w0=pi for HP
%
% Notes: L = floor(N/2) = number of second-order sections of LP analog prototype
%
%        N,Ad,wd may be obtained from DFORD, based on specifications {wp,ws,Ap,As},
%        and, then, passed directly into DFDES
%
%        Ad,wd are entered using the following conventions:
%
%        [B,A,w0] = dfdes(N,Ap,wp,0,shape, coeffs)          % match Ap at wp
%        [B,A,w0] = dfdes(N,Ap,wp,1,shape, coeffs)          % match Ap at wp
%        [B,A,w0] = dfdes(N,As,ws,2,shape, coeffs)          % match As at ws
%        [B,A,w0] = dfdes(N,[Ap,As],wp,3,shape, coeffs)     % match Ap at wp, equiripple at Ap,As
%
%        [B,A,w0] = dfdes(N,[Ap,Ab],wb,1,shape, coeffs)     % match Ab at wb, equiripple at Ap 
%        [B,A,w0] = dfdes(N,[Ab,As],wb,2,shape, coeffs)     % match Ab at wb, equiripple at As
%        [B,A,w0] = dfdes(N,[Ap,Ab,As],wb,3,shape, coeffs)  % match Ab at wb, equiripple at Ap,As
%        [B,A,w0] = dfdes(N,[Ap,As,As],ws,3,shape, coeffs)  % match As at ws, equiripple at Ap,As
%
%        substitute for built-in functions BUTTER, CHEBY1, CHEBY2, ELLIP
%
% Reference: "Lecture Notes on Elliptic Filter Design" by Sophocles J. Orfanidis, November 2006
%             available from www.ece.rutgers.edu/~orfanidi/ece521

% -----------------------------------------------------------------------------------------------
% Sophocles J. Orfanidis - November 20, 2006
% Email:   orfanidi@ece.rutgers.edu
% -----------------------------------------------------------------------------------------------

function [B,A,w0] = dfdes(N, Ad, wd, type, shape, coeffs)

if nargin==0, help dfdes; return; end
if nargin<5, disp('not enough input arguments'); return; end
if nargin==5, coeffs='4os'; end

d = size(wd,2);                   % d=1 for LP/HP, d=2 for BP/BS
a = size(Ad,2);

if d==2 & (shape=='LP' | shape=='HP'), disp('w must be 1d'); return; end
if a==3 & type<3, disp('A can be 3d only in the elliptic case'); return; end
if a==1, Ad = [Ad,Ad]; end      % handle cases a=1 and a=2 together

r = mod(N,2); L = floor(N/2);    % r=0,1 for N even,odd, L = no. of analog SOSs        

e  = sqrt(10.^(Ad/10) - 1);                                  % ripple factors
Gp = 10^(-Ad(1)/20);                                         % passband gain
H0 = (type==0 | type==2) + (type==1 | type==3) * Gp^(1-r);   % DC gain of LP analog filter

% ------ map digital frequency specs to analog ----------------------------

switch shape                   % specs matched exactly at level Ad and frequency band wd
   case 'LP'                               
      w0 = 0; c0 = 1;
      Wd = tan(wd/2);                                   % matched LP cutoff frequency wd                                                         
   case 'HP'
      w0 = pi; c0 = -1;
      Wd = cot(wd/2);                                   % matched LH cutoff frequency wd                
   case 'BP'                                             
      c0 = sin(sum(wd))/sum(sin(wd)); w0 = acos(c0);    % center freq of matched BP band
      Wd = tan(diff(wd)/2);                             % diff(wd) = matched bandwidth
   case 'BS'
      c0 = sin(sum(wd))/sum(sin(wd)); w0 = acos(c0);       
      Wd = cot(diff(wd)/2);
end

% ------ construct poles and zeros of LP analog prototype -----------------

i=(1:L)'; ui = (2*i-1)/N;

switch type
  case 0
    ep = e(1); Wp = Wd;                         % Wd,Ad represent passband Wp,Ap 
    za = ones(L,1) * Inf;                       % zeros of analog LP
    za0 = Inf;
    pa = Wp * j * exp(j*ui*pi/2) / ep^(1/N);    % poles of LP analog 
    pa0 = -Wp / ep^(1/N);
  case 1
    ep = e(1); ed = e(2);                       % remap matched Wd to passband Wp
    Wp = Wd / cosh(acosh(ed/ep)/N);             % solve C_N(Wd/Wp) = ed/ep
    v0 = asinh(1/ep) / (N*pi/2); 
    za = ones(L,1) * Inf;                            
    za0 = Inf;
    pa = Wp * j * cos((ui-j*v0)*pi/2);          % construct poles from Wp
    pa0 = -Wp * sinh(v0*pi/2);
  case 2   
    ed = e(1); es = e(2);                       % remap matched Wd to stopband Ws
    Ws = Wd * cosh(acosh(es/ed)/N);             % solve C_N(Ws/Wd) = es/ed
    v0 = asinh(es) / (N*pi/2); 
    za =  -j * Ws ./ cos(ui*pi/2);              % construct zeros from Ws
    za0 = Inf;
    pa =  -j * Ws ./ cos((ui-j*v0)*pi/2);       % construct poles from Ws
    pa0 = -Ws / sinh(v0*pi/2);        
  case 3
    if a==2, 
      ep = e(1); es = e(2);                     % match Wp, equiripple at Ap,As 
    else 
      ep = e(1); ed = e(2); es = e(3);          % match intermediate Wd
    end
    k1 = ep/es;                                 % recalculate k from N,Ap,As in order 
    k = ellipdeg(N,k1);                         % to satisfy the degree equation
    if a==2,                                    
      Wp = Wd;                                  % match passband Wp   
    else                                        % match at intermediate Wd
      ud = acde(ed/ep,k1)/N;                    % solve F_N(Wd/Wp) = ed/ep
      Wp = Wd / cde(ud,k);                      % remap matched Wd to passband Wp
    end
    v0 = -j * asne(j/ep, k1) / N;
    za = Wp * j ./(k*cde(ui,k));                % construct zeros from Wp and k
    za0 = Inf;
    pa = Wp * j * cde(ui-j*v0, k);              % construct poles from Wp and k
    pa0 = Wp * j * sne(j*v0, k); 
end             

% ------ poles, zeros, and coefficients of hat second-order sections ------

if shape=='BS', q=-1; else q=1; end                 % q=1 for LP/HP/BP, q=-1 for BS

zh  = (1./za + 1)./(1./za - 1);                     % z_hat zeros, note 1/za=0 if za=Inf
zh0 = (1./za0 + 1)./(1./za0 - 1);
ph  = (1 + pa)./(1 - pa);                           % p_hat poles
ph0 = (1 + pa0)/(1 - pa0);

G = diag(abs((1-ph)./(1-zh)));                      % gains of hat-SOS are G^2
G0 = (1-r*ph0)/(1-r*zh0);                           % unity gain for even N (r=0)

Bh = G^2 * [ones(L,1), -2*real(zh), abs(zh).^2];    % hat-coefficients, 2nd-order sections
Ah = [ones(L,1), -2*real(ph), abs(ph).^2];
Bh = [H0*G0*[1, -r*real(zh0), 0]; Bh];              % prepend 1st-order section (r=1), or gain (r=0)
Ah = [[1, -r*real(ph0), 0]; Ah];

if c0==1 | c0==-1 	       % LP or HP cases, w0=0 or w0=pi

  B4 = Bh;                     % B4,A4 are second-order sections for LP/HP
  A4 = Ah;                                  
  B4(:,2) = c0*B4(:,2);        % change sign for HP case (because zhat=-z)
  A4(:,2) = c0*A4(:,2);        
  B2 = B4;                     % '4os' and 'sos' output formats are identical for LP/HP
  A2 = A4;

else                           % BP or BS cases, 0 < w0 <pi

  %  ------ poles, zeros, and coefficients of fourth-order sections --------

  z1 =  c0*(1+q*zh)/2 + sqrt(c0^2*(1+q*zh).^2 - 4*q*zh)/2;       % "plus" group of zeros & poles
  p1 =  c0*(1+q*ph)/2 + sqrt(c0^2*(1+q*ph).^2 - 4*q*ph)/2;     
  z01 = c0*(1+q*zh0)/2 + sqrt(c0^2*(1+q*zh0).^2 - 4*q*zh0)/2;     
  p01 = c0*(1+q*ph0)/2 + sqrt(c0^2*(1+q*ph0).^2 - 4*q*ph0)/2; 

  z2 = (c0-z1)./(1-c0*z1);                                       % "minus" group of zeros & poles
  p2 = (c0-p1)./(1-c0*p1);                         
  z02 = (c0-z01)/(1-c0*z01);                       
  p02 = (c0-p01)/(1-c0*p01);                       

  b1 = real(z1); r1 = abs(z1).^2; b2 = real(z2); r2 = abs(z2).^2;  % defined for convenience
  a1 = real(p1); R1 = abs(p1).^2; a2 = real(p2); R2 = abs(p2).^2; 

  B4 = G^2 * [ones(L,1), -2*(b1+b2), (r1 + r2 + 4*b1.*b2), -2*(b1.*r2 + b2.*r1), r1.*r2];  
  A4 = [ones(L,1), -2*(a1+a2), (R1 + R2 + 4*a1.*a2), -2*(a1.*R2 + a2.*R1), R1.*R2];

  B4 = [H0*G0*[1, -r*real(z01+z02), r*real(z01*z02), 0, 0]; B4];   % p01,p02 are either both
  A4 = [[1, -r*real(p01+p02), r*real(p01*p02), 0, 0]; A4];         % real or conjugate pairs

  % ------ poles, zeros, and coefficients of final second-order sections ----

  z = [z1; z2];                                     % concatenate zeros and poles
  p = [p1; p2];  

  maxp = max(abs((p)))

  g = diag([diag(G); diag(G)]);                     % split gain equally between z1 and z2 factors

  B2 = g * [ones(2*L,1), -2*real(z), abs(z).^2];    % second-order sections
  A2 = [ones(2*L,1), -2*real(p), abs(p).^2];
  B2 = [B4(1,1:3); B2];
  A2 = [A4(1,1:3); A2];

end          

% ------ direct-form coefficients -----------------------------------------

if r==0,
   b = B2(1,1); a = 1; 
else
   b = B2(1,:); a = A2(1,:);
end

for i=2:size(B2,1)
   b = conv(b,B2(i,:)); 
   a = conv(a,A2(i,:));
end

% ----- select output format -----------------------------------------------

switch coeffs
  case '4os'                   % fourth-order sections
    B = B4; A = A4;
  case 'sos'                   % second-order sections
    B = B2; A = A2;
  case 'hsos'                  % hat-second-order sections, w0 output is also needed
    B = Bh; A = Ah; 
  case 'dir'                   % direct form
    B = b; A = a;             
end
