% dford.m  -  digital filter order determination
% 
% Usage: [N,Ad,wd] = dford(wp,ws,Ap,As,type,match)
%
% wp,ws = passband and stopband frequencies in rads/sample, (1d for LP/HP, 2d for BP/BS)
% Ap,As = passband and stopband attenuations in dB
% type  = 0,1,2,3 for Butterworth, Chebyshev-1, Chebyshev-2, Elliptic
% match = 'p', 's', if omitted it defaults to 'p' for type=0,1,3, 's' for type=2
%
% N  = filter order of LP analog prototype
% Ad = attenuation in dB to be matched exaclty at design frequency wd, (1d, 2d, or, 3d, see notes below)
% wd = design frequency (1d or 2d) that must be matched exactly
%
% it determines the type LP/HP/BP/BS from wp,ws using the following conventions:
%
%    match='p' matches passband specs exactly
%    match='s' matches stopband specs exactly
%
%    wp,ws are in radians/sample, e.g., wp = 2*pi*fp/fsamp
%
%    LP: wp,ws are scalars with wp < ws
%    HP: wp,ws are scalars with wp > ws
%    BP: wp = [wp1,wp2], ws = [ws1,ws2], with ws1 < wp1 < wp2 < ws2
%    BS: wp = [wp1,wp2], ws = [ws1,ws2], with wp1 < ws1 < ws2 < wp2
%
%    N,Ad,wd may be passed to DFDES to design the filter: [B,A] = dfdes(N,Ad,wd,type,shape,coeffs)
%
%    Ad,wd are generated as follows (and can be passed directly to DFDES):
%
%    type = 0, Ad = Ap,         wd = wp     match Ap at wp, default
%    type = 1, Ad = Ap,         wd = wp     match Ap at wp, default
%    type = 1, Ad = [Ap,As],    wd = ws     match As at ws, and equiripple at Ap
%    type = 2, Ad = As,         wd = ws     match As at ws, default
%    type = 2, Ad = [Ap,As],    wd = wp     match Ap at wp, and equiripple at As
%    type = 3, Ad = [Ap,As],    wd = wp     match Ap at wp, and equiripple at Ap,As, default  
%    type = 3, Ad = [Ap,As,As], wd = ws     match As at ws, and equiripple at Ap,As
%
%    substitute for built-in functions BUTTORD, CHEB1ORD, CHEB2ORD, ELLIPORD
%
% Reference: "Lecture Notes on Elliptic Filter Design" by Sophocles J. Orfanidis, November 2006
%             available from www.ece.rutgers.edu/~orfanidi/ece521

% -----------------------------------------------------------------------------------------------
% Sophocles J. Orfanidis - November 20, 2006
% Email:   orfanidi@ece.rutgers.edu
% -----------------------------------------------------------------------------------------------

function [N,Ad,wd] = dford(wp, ws, Ap, As, type, match)

if nargin==0, help dford; return; end
if nargin<5, disp('not enough input arguments'); return; end
if nargin==5, match = 'd'; end   % default matching

% ------ construct Ad,wd ------------------------------------------------------

switch match
  case 'p'                            % match passband
     switch type
        case {0,1}
           Ad = Ap; wd = wp;
        case 2
           Ad = [Ap,As]; wd = wp;
        case 3
           Ad = [Ap,As]; wd = wp;
     end
  case 's'                            % match stopband
     switch type
        case 0
           Ad = As; wd = ws;
        case 1
           Ad = [Ap,As]; wd = ws;
        case 2
           Ad = As; wd = ws;
        case 3
           Ad = [Ap,As,As]; wd = ws;
     end
  case 'd'                            % default matching
     switch type
        case {0,1}
           Ad = Ap; wd = wp;
        case 2
           Ad = As; wd = ws;
        case 3
           Ad = [Ap,As]; wd = wp;
     end
end

% ------ map specs to equivalent analog LP specs  -----------------------------

d = size(wp,2);                                     % d=1 for LP/HP, d=2 for BP/BS

if d==1 & (wp < ws)                                 % LP case
   Wp = tan(wp/2); 
   Ws = tan(ws/2);
elseif d==1 & (wp > ws)                             % HP case
   Wp = cot(wp/2);
   Ws = cot(ws/2);
elseif d==2 & (ws(1) < wp(1)) & (wp(2) < ws(2))     % BP case
   w0 = acos(sin(sum(wd))/sum(sin(wd)));            % center frequency at matched band
   Wd = tan(diff(wd)/2);                            % analog bandwidth of matched band
   if wd==wp,                                       % matching passband
     Wp = Wd;
     Ws = min(abs((cos(w0) - cos(ws))./sin(ws)));
   else                                             % matching stopband
     Wp = max(abs((cos(w0) - cos(wp))./sin(wp)));
     Ws = Wd;
   end
elseif d==2 & (wp(1) < ws(1)) & (ws(2) < wp(2))     % BS case
   w0 = acos(sin(sum(wd))/sum(sin(wd)));            % center frequency at matched band
   Wd = cot(diff(wd)/2);                            % analog bandwidth of matched band
   if wd==wp,                                       % matching passband
     Wp = Wd;
     Ws = min(abs(sin(ws)./(cos(w0) - cos(ws))));
   else                                             % matching stopband
     Ws = Wd;
     Wp = max(abs(sin(wp)./(cos(w0) - cos(wp))));
   end
else
   disp('inconsistent specs');
   return;
end

ep = sqrt(10^(Ap/10) - 1);                          % ripple factors
es = sqrt(10^(As/10) - 1);

k = Wp/Ws; k1 = ep/es;                              % design parameters

switch type                                         % determine order N
  case 0                                            % Butterworth
     N = ceil(log(1/k1) / log(1/k)); 
  case {1,2}                                        % Chebyshev 1 & 2
     N = ceil(acosh(1/k1) / acosh(1/k)); 
  case 3                                            % elliptic
    [K,Kp] = ellipk(k); 
    [K1,K1p] = ellipk(k1);
    N = ceil((K1p/K1)/(Kp/K));
end






